/*
// File:		ImapCallback.cpp
//
// Version:		mail.it library 1.0
//
// Copyright:	(C) 2007 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#include PCH_FILE_NAME
#include "ImapCallback.hpp"
#include "StreamUtils.hpp"

int
ImapCallback::OnOperation(
		Emai::ImapSession& inSession,
		EmaiUint32 inOperationCode
	)
{
	HWND hwnd = NULL;
	EmaiUint32 statSize = sizeof(HWND);
	inSession.GetUserData(ImapStatusText, &hwnd, statSize);
	CWnd* wnd = CWnd::FromHandle(hwnd);
	CString statusTexts[] = {
		_T("Connecting to server..."),
		_T("Successfully connected to server."),
		_T("Authenticating on server..."),
		_T("Successfully authenticated on server."),
		_T("Creating folder..."),
		_T("Folder successfully created."),
		_T("Deleting folder..."),
		_T("Folder successfully deleted."),
		_T("Subsribing folder..."),
		_T("Folder successfully subscribed."),
		_T("Unsubsribing folder..."),
		_T("Folder successfully unsubscribed."),
		_T("Renaming folder..."),
		_T("Folder successfully renamed."),
		_T("Retrieving folder list..."),
		_T("Folder list successfully retrieved."),
		_T("Examining folder..."),
		_T("Folder successfully examined."),
		_T("Retrieving folder status..."),
		_T("Folder status successfully retrieved."),
		_T("Selecting folder..."),
		_T("Folder successfully selected."),
		_T("Closing folder..."),
		_T("Folder successfully closed."),
		_T("Checking..."),
		_T("Checked."),
		_T("Appending message..."),
		_T("Message successfully appended."),
		_T("Searching messages..."),
		_T("Message search finished."),
		_T("Fetching messages..."),
		_T("Messages successfully fetched."),
		_T("Copying messages..."),
		_T("Messages successfully copied."),
		_T("Storing messages data..."),
		_T("Messages data successfully stored."),
		_T("Expunging messages..."),
		_T("Messages successfully expunged."),
		_T("Sending NOOP."),
		_T("NOOP successfully sent."),
		_T("Closing connection."),
		_T("Connection successfully closed.")
	};

	wnd->SetWindowText(statusTexts[inOperationCode]);
	return EmaiErrNoErr;
}

int
ImapCallback::OnDataSent(
		Emai::ImapSession& inSession,
		EmaiUint32 inSentSize,
		EmaiUint32 inTotalSize
	)
{
	HWND hwnd = NULL;
	EmaiUint32 wndSize = sizeof(HWND);
	inSession.GetUserData(ImapPercentText, &hwnd, wndSize);
	CWnd* wnd = CWnd::FromHandle(hwnd);
	int percent = 0;
	int curPercent = (100 * inSentSize) / inTotalSize;
	EmaiUint32 numSize = sizeof(EmaiUint32);
	inSession.GetUserData(ImapCurrentPercent, &percent, numSize);
	if (percent != curPercent)
	{
		CString message;
		message.Format(_T("%d %%"), curPercent);
		wnd->SetWindowText(message);
		inSession.SetUserData(ImapCurrentPercent, &curPercent, numSize);
	}
	return EmaiErrNoErr;
}

int
ImapCallback::OnDataReceived(
		Emai::ImapSession& inSession,
		EmaiUint32 inReceivedSize,
		EmaiUint32 inTotalSize
	)
{
	HWND hwnd = NULL;
	EmaiUint32 wndSize = sizeof(HWND);
	inSession.GetUserData(ImapPercentText, &hwnd, wndSize);
	CWnd* wnd = CWnd::FromHandle(hwnd);
	int percent = 0;
	int curPercent = (100 * inReceivedSize) / inTotalSize;
	EmaiUint32 numSize = sizeof(EmaiUint32);
	inSession.GetUserData(ImapCurrentPercent, &percent, numSize);
	if (percent != curPercent)
	{
		CString message;
		message.Format(_T("%d %%"), curPercent);
		wnd->SetWindowText(message);
		inSession.SetUserData(ImapCurrentPercent, &curPercent, numSize);
	}
	return EmaiErrNoErr;
}

int
ImapCallback::OnServerError(
		Emai::ImapSession& inSession,
		EmaiConstCharPtr inServerReply,
		EmaiUint32 /*inReplyCode*/
	)
{
	HWND hwnd = NULL;
	EmaiUint32 wndSize = sizeof(HWND);
	inSession.GetUserData(ImapStatusText, &hwnd, wndSize);
	CWnd* wnd = CWnd::FromHandle(hwnd);
	CString message;
	message.Format(_T("The server error encountered was: %S"), inServerReply);
	wnd->SetWindowText(message);

	return EmaiErrNoErr;
}
