/*!
// @file		ImapGUIDlg.hpp
//
// @brief		Dialog class
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2007 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#pragma once
#include "afxwin.h"

#include <EmaiIMAP.hpp>
#include "ImapResponseHandler.hpp"
#include "ImapCallback.hpp"

class ImapFolder;
class MessagePartEnumerator;

/*!
// @class	CImapGUIDlg
// @brief	Imap sample dialog class.
*/
class CImapGUIDlg : public CDialog
{
public:
	/*!
	// @brief	Standart constructor
	//
	// @param	pParent - pointer to parent window
	*/
	CImapGUIDlg(CWnd* pParent = NULL);

	/*!
	// @brief	Standart destructor
	*/
	~CImapGUIDlg();

	enum { IDD = IDD_IMAPGUI_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);

private:
	ImapCallback mStatusCallback;
	ImapResponseHandler mImapResponseHandler;
	Emai::ImapSession mImapSession;
	UINT mTimerID;
	EmaiUint32 mLastRetriveMsgIndex;
    CString mLastRetriveMsgItem;

	/*!
	// @brief	Connects to server and creates session.
	//
	// @return	connection status
	*/
	bool ConnectSession();

	/*!
	// @brief	Fetches data from server.
	//
	// @param	inFetchAttr - message attributes
	//
	// @param	attributeCount - attributes count
	//
	// @param   msgSequence - message sequence object link
	//
	// @return	returns true if message fetched successful
	*/
	bool FetchData(EmaiFetchMessageAttribute inFetchAttr[], EmaiUint32 attributeCount, Emai::ImapSequenceSet& msgSequence);

	/*!
	// @brief	Formats error messages.
	//
	// @param	inException - reference to exception class
	*/
	void FormatError(Emai::Exception& inException);

	/*!
	// @brief	Returns formatted text string for message dialog's fields.
	//
	// @param	inFieldType - field type
	//
	// @param	inEnumerator - reference to enumerator class
	//
	// @return	string with formatted text
	*/
	CString FormatTextForTextField(EmaiOptions inFieldType, 
		MessagePartEnumerator& inEnumerator) const;

	/*!
	// @brief	Reloads mailboxes list.
	//
	// @param	inMailbox - index of mailbox
	//
	*/
	void LoadMessages(int inMailbox);

	/*!
	// @brief	Resets connection.
	//
	*/
	void ResetConnection();

	/*!
	// @brief	Gets active mailbox.
	//
	// @return	pointer to ImapFolder class
	*/
	ImapFolder* GetActiveMailbox();

// Implementation
protected:
	HICON m_hIcon;

	/*!
	// @brief	Initialize dialog function.
	//
	// @return	initialization result
	*/
	virtual BOOL OnInitDialog();

	/*!
	// @brief	Destroy dialog function. Calls before destructor.
	//
	*/
	virtual void OnDestroy();
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();

	/*!
	// @brief	Calls every 60 second and sends noop message to server as soon as WM_TIMER message received.
	//
	// @param	nIDEvent - timer id
	*/
	afx_msg void OnTimer(UINT nIDEvent);

	DECLARE_MESSAGE_MAP()

	CString mMailServer;
	CString mAccount;
	CString mPassword;
	CString mStatusText;
	CString mLicenseKey;

	CListCtrl mMailboxesList;
	CListCtrl mMessagesList;

	CString mStatusPerc;
	UINT mPort;
	BOOL mUseSsl;
	CComboBox mAuthType;
	CButton mAnotherServer;
	CEdit mMailServerEdit;
	CEdit mAccountEdit;
	CEdit mPasswordEdit;
	CEdit mPortEdit;
	CButton mUseSslButton;
	CButton mGetMailboxes;
	CEdit mLicenseKeyEdit;

	/*!
	// @brief	Calls when user clicks on Get mailboxes button.
	*/
	afx_msg void OnBnClickedButtonGetmailboxes();

	/*!
	// @brief	Calls when user clicks on Add folder (+) button.
	*/
	afx_msg void OnBnClickedButtonAddMailbox();

	/*!
	// @brief	Calls when user clicks on Delete folder (-) button.
	*/
	afx_msg void OnBnClickedButtonDeleleteMailbox();

	/*!
	// @brief	Calls when user clicks on Delete message (-) button.
	*/
	afx_msg void OnBnClickedButtonDeleteMessage();

	/*!
	// @brief	Calls when user doubleclicks in message list.
	*/
	afx_msg void OnLvnItemActivateListMessages(NMHDR *pNMHDR, LRESULT *pResult);

	/*!
	// @brief	Calls when user clicks on Add message (+) button.
	*/
	afx_msg void OnBnClickedButtonAddMessage();

	/*!
	// @brief	Calls when user clicks on mailbox in mailboxes list.
	*/
	afx_msg void OnItemchangedListMailboxes(NMHDR *pNMHDR, LRESULT *pResult);

	/*!
	// @brief	Calls when user clicks on Expunge button.
	*/
	afx_msg void OnBnClickedButtonExpunge();

	/*!
	// @brief	Functions for virtual list manipulations
	*/
	afx_msg void OnCustomDrawMailboxes(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnGetMailboxesDispInfoList(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnCustomDrawMessages(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnGetMessagesDispInfoList(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnLvnOdcachehintListMessages(NMHDR *pNMHDR, LRESULT *pResult);

	/*!
	// @brief	Calls when user clicks on Another server button.
	*/
	afx_msg void OnBnClickedButtonAnother();

	/*!
	// @brief	Calls when ending renaming mailbox.
	*/
	afx_msg void OnLvnEndlabeleditListMailboxes(NMHDR *pNMHDR, LRESULT *pResult);

	/*!
	// @brief	Calls when user clicks on Rename button.
	*/
	afx_msg void OnBnClickedButtonRename();

	/*!
	// @brief	Calls when user clicks on Undelete button.
	*/
	afx_msg void OnBnClickedButtonUndelete();

	/*!
	// @brief	Calls when user kills focus on licence textbox.
	*/
	afx_msg void OnEnKillfocusEditLickey();

	/*!
	// @brief	Calls when user sets focus on licence textbox.
	*/
	afx_msg void OnEnSetfocusEditLickey();
};
