/*
// File:		ReceiveMessages.cpp
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2006 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#include PCH_FILE_NAME
#include <EmaiPOP3.hpp>
#include <EmaiMessage.hpp>
#include <iostream>
#include "POP3Callback.hpp"
#include "MessagePartEnumerator.hpp"
#include "PrintMessage.hpp"
#include "StreamUtils.hpp"

// Settings

EmaiUint32 sTimeout				= 10;

EmaiUniChar sMailServer[]	= {'m', 'a', 'i', 'l', '.', 'm', 'y', 'd', 'o', 'm', 'a', 'i', 'n', '.', 'c', 'o', 'm', '\0'};
EmaiUniChar sAccount[]		= {'m', 'y', 'a', 'c', 'c', 'o', 'u', 'n', 't', '\0'};
EmaiUniChar sPassword[]		= {'m', 'y', 'p', 'a', 's', 's', 'w', 'o', 'r', 'd', '\0'};

EmaiUniChar sHistoryFileName[]	= {'H', 'y', 's', 't', 'o', 'r', 'y', '\0'};

short sPort						= EmaiDefaultPop3Port;

unsigned int sMaxDayMessageLag	= 1;

void
ReciveMessage()
{
	Pop3Callback statusCallback;

	Emai::Pop3Session pop3Session(statusCallback, sTimeout);

	try
	{
		pop3Session.Connect(sMailServer, EmaiDefaultPop3Port, EmaiNullOptions);

		pop3Session.Authenticate(sAccount, sPassword);

		Emai::MessageInfoList inMessageInfoList(sHistoryFileName);

		pop3Session.RetrieveMessageList(inMessageInfoList);
		
		EmaiUint32 currentMessageNumber = 0;
		EmaiUint32 messageCount = inMessageInfoList.GetCount();

		while (currentMessageNumber < messageCount)
		{
			EmaiMessageInfo outInfo;
			inMessageInfoList.GetInfo(currentMessageNumber, outInfo);

			std::cout << std::endl;
			std::cout << "********************************************" << std::endl;
			std::cout << "Downloading message " << outInfo.messageNumber << std::endl;
			std::cout << "********************************************" << std::endl;
			std::cout << std::endl;
			
			Emai::Message message = pop3Session.RetrieveMessage(outInfo.messageNumber);

			std::cout << std::endl;
			std::cout << "********************************************" << std::endl;
			std::cout << "Message " << outInfo.messageNumber << std::endl;
			std::cout << "********************************************" << std::endl;
			std::cout << std::endl;

			MessagePartEnumerator messagePartEnumerator(message);
			messagePartEnumerator.Enumerate();

			PrintMessage(messagePartEnumerator);

			inMessageInfoList.MarkMessageReceived(++currentMessageNumber);
		}
		
		inMessageInfoList.EvaluateSublist(sMaxDayMessageLag, EmaiMessagesForDeletion);
        		
		std::cout << std::endl;
		std::cout << "********************************************" << std::endl;
		std::cout << "Deleting messages" << std::endl;
		std::cout << "********************************************" << std::endl;
		std::cout << std::endl;

		currentMessageNumber = 0;
		messageCount = inMessageInfoList.GetCount(EmaiMessagesForDeletion);

		while (currentMessageNumber < messageCount)
		{
			EmaiMessageInfo outInfo;
			inMessageInfoList.GetInfo(currentMessageNumber, outInfo);

			pop3Session.DeleteMessage(outInfo.messageNumber);

			std::cout << "Message " << outInfo.messageNumber << " deleted." << std::endl;
			currentMessageNumber++;
		}

		std::cout << std::endl;
		std::cout << "Messages successfully downloaded and deleted." << std::endl;
		
		pop3Session.Disconnect();
	}

	catch (Emai::Exception& exception)
	{
		std::cout << "Error number " << exception.GetErrorCode() << "." << std::endl;
	}
}

int
main()
{
	ReciveMessage();
	
	std::cout << "Done! Press <enter>." << std::endl;
	std::cin.get();

	return 0;
}
